package com.habii.treasurehunt.framework;

import org.eclipse.emf.common.util.EList;

import com.habii.treasurehunt.framework.ui.GameUI;
import com.habii.treasurehunt.framework.ui.MapPanel;
import com.habii.treasurehunt.framework.ui.map.PlayerWayPoint;
import com.habii.treasurehunt.framework.ui.map.PostWayPoint;
import com.habii.treasurehunt.treasurehunt.Level;
import com.habii.treasurehunt.treasurehunt.Mobility;
import com.habii.treasurehunt.treasurehunt.Order;
import com.habii.treasurehunt.treasurehunt.Player;
import com.habii.treasurehunt.treasurehunt.Post;
import com.habii.treasurehunt.treasurehunt.Temporal;
import com.habii.treasurehunt.treasurehunt.impl.ConfigImpl;

public class EngineConfig extends ConfigImpl {

	public EngineConfig() {
	}
	
	// Test for HIGH = Success
	// Test for Medium = Success
	// test for Low = Success
	// This effects only Temporal axis
	public boolean foundInTime(EnginePlayer player, Post post){
		Temporal temporalAxis = player.getWorld().getConfig().getTemporalAxis();
		Level temporal = temporalAxis.getLevel();
		
		long deadline = player.getDeadline();

		long timeSpent = System.currentTimeMillis()-player.getStopWatch();
		
		switch (temporal) {
		case HIGH:// strict timeout
			if(timeSpent>deadline){
				System.out.println(player.TIMEOUT_ERROR);
				return false;
			}
			break;
		case MEDIUM:
			if(temporalAxis.getPosts().contains(post)){
				if(timeSpent>deadline){
					System.out.println(player.TIMEOUT_ERROR);
					return false;
				}
			}
			break;
		case LOW:
			break;
		default:
			break;
		}
		return true;
	}
	
	// Test for HIGH = Success
	// Test for Medium = Success
	// test for Low = Success
	// This effects only Mobility axis
	public boolean foundInOrder(EnginePlayer player, Post post){
		Mobility mobilityAxis = player.getWorld().getConfig().getMobilityAxis();
		Level mobility = mobilityAxis.getLevel();
		EList<Order> ordering = mobilityAxis.getOrdering();
		switch (mobility) {
		case HIGH:
			for (Order order : ordering) {
				if(order.getP2().equals(post)){
					if(!player.getVisited().contains(order.getP1())){
						System.out.println(player.ORDER_ERROR);
						return false;
					}
				}
			}
			break;
		case MEDIUM:
			for (Order order : ordering) {
				if(order.getP2().equals(post)){
					if(!player.getVisited().contains(order.getP1())){
						System.out.println(player.ORDER_ERROR);
						return false;
					}
				}
			}
			
			break;
		case LOW:
			break;
		default:
			break;
		}
		return true;
	}
	
	
	public int calculatePoints(EnginePlayer player){
		int points = -1;
		if (goal == Level.LOW) {
			points = player.getVisited().size();
		} else if (goal == Level.MEDIUM) {
			points = player.getVisited().size();
			points += player.getSolved().size();
		} else if (goal == Level.HIGH) {
			points = player.getSolved().size();
		} else {
			points = 0;
		}
		return points;
	}
	
	
	public void repaintMap(EngineWorld world, GameUI gui){
		gui.getMap().cleanWayPoints(); // start empy every time
		if (goal == Level.HIGH) {
			repainAllPostsPosition(world, gui.getMap());
		} else {
			repainOnlyVisistedPostsPosition(world, gui.getMap());
		}
		repainAllPlayerPosition(world, gui.getMap());
	}
	
	public void repaintEditor(EnginePlayer actor, GameUI gui){
		gui.updateAll(actor);
	}
	
	
	private void repainAllPlayerPosition(EngineWorld world, MapPanel map) {
		for (Player player : world.getAllPlayers()) {
			map.addWayPoint(new PlayerWayPoint((EnginePlayer) player));
		}
	}

	private void repainOnlyVisistedPostsPosition(EngineWorld world, MapPanel map) {
		for (Player p : world.getAllPlayers()) {
			EList<Post> visited = p.getVisited();
			if (visited != null) {
				for (Post post : visited) {
					map.addWayPoint(new PostWayPoint(post));
				}
			}
		}
	}

	private void repainAllPostsPosition(EngineWorld world, MapPanel map) {
		for (Post post : world.getAllPosts()) {
			map.addWayPoint(new PostWayPoint(post));
		}
	}
}
