package com.habii.treasurehunt.framework;

import javax.measure.unit.NonSI;

import org.jscience.geography.coordinates.LatLong;

import com.habii.treasurehunt.framework.util.TimeUtils;
import com.habii.treasurehunt.treasurehunt.Config;
import com.habii.treasurehunt.treasurehunt.Direction;
import com.habii.treasurehunt.treasurehunt.DistanceToClosestPost;
import com.habii.treasurehunt.treasurehunt.Level;
import com.habii.treasurehunt.treasurehunt.Post;
import com.habii.treasurehunt.treasurehunt.Task;
import com.habii.treasurehunt.treasurehunt.Temporal;
import com.habii.treasurehunt.treasurehunt.impl.PlayerImpl;

public class EnginePlayer extends PlayerImpl {
	public final String ORDER_ERROR = "WRONG ORDER";
	public final String TIMEOUT_ERROR = "Timeout!";
	public final double DEFAULT_DISTANCE_JUMP = 0.0001;
	private long start;
	private long end;
	private long stopWatch;
	private boolean finished = false;

	public EnginePlayer() {
		super();
		end = -1;
		start = -1;
		stopWatch= -1;
	}


	public boolean isFinished() {
		return finished;
	}

	public long getStart() {
		return start;
	}


	public long getEnd() {
		return end;
	}


	public long getStopWatch() {
		return stopWatch;
	}
	
	
	public int caculatePoints() {
		Config config = world.getConfig();
		return ((EngineConfig) config).calculatePoints(this);
	}

	public void setLocation(double x, double y) {
		if (world != null) {
			this.currentPosition = LatLong.valueOf(x, y, NonSI.DEGREE_ANGLE);
		}
	}

	public void move(double distance) {
		EngineWorld world = (EngineWorld) getWorld();
		if (world != null) {
			double x = getCurrentPosition().latitudeValue(NonSI.DEGREE_ANGLE);
			double y = getCurrentPosition().longitudeValue(NonSI.DEGREE_ANGLE);
			switch (direction) {
			case NORTH:
				setLocation(x + distance, y);
				break;
			case SOUTH:
				setLocation(x - distance, y);
				break;
			case WEST:
				setLocation(x, y - distance);
				break;
			case EAST:
				setLocation(x, y + distance);
				break;
			default:
				break;
			}
		}
	}

	public void doMove(String direction) {
		if (direction.equals("W")) {
			setDirection(Direction.NORTH);
			move(DEFAULT_DISTANCE_JUMP);
		} else if (direction.equals("S")) {
			setDirection(Direction.SOUTH);
			move(DEFAULT_DISTANCE_JUMP);
		} else if (direction.equals("A")) {
			setDirection(Direction.WEST);
			move(DEFAULT_DISTANCE_JUMP);
		} else if (direction.equals("D")) {
			setDirection(Direction.EAST);
			move(DEFAULT_DISTANCE_JUMP);
		} else {
			// ?
		}
	}

	public Post calculateDistanceToClosestPost() {
		
		double latitude = getCurrentPosition()
				.latitudeValue(NonSI.DEGREE_ANGLE);
		double longtitude = getCurrentPosition().longitudeValue(
				NonSI.DEGREE_ANGLE);

		Post p = found(latitude, longtitude);
		
		if(p!=null){
			setDistanceToClosestPost(DistanceToClosestPost.FOUND);
			if(getVisited().contains(p)){
				return null;
			} else {
				EngineConfig config = (EngineConfig) world.getConfig();
				boolean foundInTime = config.foundInTime(this, p);
				boolean foundInOrder = config.foundInOrder(this, p);
				if(foundInOrder && foundInTime){
					if (!getVisited().contains(p)) {
						getVisited().add(p);
						if (!p.getVisitedBy().contains(this)) {
							p.getVisitedBy().add(this);
						}
						if(config.getGoal()==Level.LOW){
							resetDeadline(p);							
						}
						if(hasCompeleted()){
							end = System.currentTimeMillis();
							finished=true;
						}
						return p;
					}
				}
				
			}
			return null;
		} else if (hot(latitude, longtitude)){
			setDistanceToClosestPost(DistanceToClosestPost.HOT);
			return null;
		} else if (warm(latitude, longtitude)){
			setDistanceToClosestPost(DistanceToClosestPost.WARM);
			return null;
		}  else if (lukewarm(latitude, longtitude)){
			setDistanceToClosestPost(DistanceToClosestPost.LUKEWARM);
			return null;
		} else{
			setDistanceToClosestPost(DistanceToClosestPost.COLD);
			return null;
		}
		
	}

	public void resetDeadline(Post post) {
		Temporal temporalAxis = getWorld().getConfig().getTemporalAxis();
		if(temporalAxis.getLevel()==Level.MEDIUM){
			if(temporalAxis.getPosts().contains(post)){
				stopWatch = System.currentTimeMillis();
			}
		} else {
			stopWatch = System.currentTimeMillis();
		}
	}


	private Post found(double latitude, double longtitude) {
		for (Post p : ((EngineWorld) world).getAllPosts()) {
			double postLatitude = p.getPosition().latitudeValue(
					NonSI.DEGREE_ANGLE);
			double postLongtitude = p.getPosition().longitudeValue(
					NonSI.DEGREE_ANGLE);

			double dLat = latitude - postLatitude;
			double dLong = longtitude - postLongtitude;
			double distance = DEFAULT_DISTANCE_JUMP;
			if (Math.abs(dLat) <= distance && Math.abs(dLong) <= distance) {
				return p;
			}
		}
		return null;
	}

	private boolean hot(double latitude, double longtitude) {
		for (Post p : ((EngineWorld) world).getAllPosts()) {
			double postLatitude = p.getPosition().latitudeValue(
					NonSI.DEGREE_ANGLE);
			double postLongtitude = p.getPosition().longitudeValue(
					NonSI.DEGREE_ANGLE);
			double dLat = latitude - postLatitude;
			double dLong = longtitude - postLongtitude;
			double distance = DEFAULT_DISTANCE_JUMP * 5;
			if (Math.abs(dLat) <= distance && Math.abs(dLong) <= distance) {
				return true;
			}
		}
		return false;
	}

	private boolean warm(double latitude, double longtitude) {
		for (Post p : ((EngineWorld) world).getAllPosts()) {
			double postLatitude = p.getPosition().latitudeValue(
					NonSI.DEGREE_ANGLE);
			double postLongtitude = p.getPosition().longitudeValue(
					NonSI.DEGREE_ANGLE);
			double dLat = latitude - postLatitude;
			double dLong = longtitude - postLongtitude;
			double distance = DEFAULT_DISTANCE_JUMP * 10;
			if (Math.abs(dLat) <= distance && Math.abs(dLong) <= distance) {
				return true;
			}
		}
		return false;
	}

	private boolean lukewarm(double latitude, double longtitude) {
		for (Post p : ((EngineWorld) world).getAllPosts()) {
			double postLatitude = p.getPosition().latitudeValue(
					NonSI.DEGREE_ANGLE);
			double postLongtitude = p.getPosition().longitudeValue(
					NonSI.DEGREE_ANGLE);
			double dLat = latitude - postLatitude;
			double dLong = longtitude - postLongtitude;
			double distance = DEFAULT_DISTANCE_JUMP * 20;
			if (Math.abs(dLat) <= distance && Math.abs(dLong) <= distance) {
				return true;
			}
		}
		return false;
	}

	public long getDeadline(){
		Temporal temporalAxis = world.getConfig().getTemporalAxis();
		long tid = 0;
		if(temporalAxis.getDeadline()!=null){
			int days = temporalAxis.getDeadline().getDays();
			int hours = temporalAxis.getDeadline().getHours();
			int minutes = temporalAxis.getDeadline().getMinutes();
			int seconds = temporalAxis.getDeadline().getSeconds();
			tid += (seconds*TimeUtils.ONE_SECOND);
			tid += (minutes*TimeUtils.ONE_MINUTE);
			tid += (hours*TimeUtils.ONE_HOUR);
			tid += (days*TimeUtils.ONE_DAY);
		}
		return tid;
	}
	
	// solved (all tasks)/found all posts
	public boolean hasCompeleted(){
		EngineWorld world = (EngineWorld)getWorld();
		Level goal = world.getConfig().getGoal();
		switch (goal) {
		case HIGH:
			if(getSolved().size()==world.getAllTasks().size()){
				return true;
			}
			break;
		case MEDIUM:
			if(getSolved().size()==world.getAllTasks().size()){
				return true;
			}
			break;
		case LOW:
			if(getVisited().size()==world.getAllPosts().size()){
				return true;
			}
		default:
			break;
		}
		return false;
	}
	
	public String timeAgo(long milliseconds){
		return TimeUtils.millisToLongDHMS(milliseconds);
	}

	// return milliseconds
	public long calculateTotalTime() {
		if(start==-1){
			return -1;
		}
		if(end!=-1){
			return end - start;			
		} else {
			return System.currentTimeMillis() - start;			
		}
	}


	public void solved(Task t) {
		if(!getSolved().contains(t)){
			getSolved().add(t);
			
			Post p = ((EngineWorld) world).getPost(t);
			
			resetDeadline(p);
			if(hasCompeleted()){
				end = System.currentTimeMillis();
				finished=true;
			}
			
		}
	}


	public long calculateStopWatch() {
		if(stopWatch==-1){
			return -1;
		}
		
		return getDeadline() - (System.currentTimeMillis() - stopWatch);
	}
	
	
	public void start(){
		if(start==-1){
			start = System.currentTimeMillis();
		}
		
		if(stopWatch==-1){
			stopWatch= System.currentTimeMillis();
		}
	}
}
