package com.habii.treasurehunt.framework;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import com.habii.treasurehunt.framework.ui.GameUI;
import com.habii.treasurehunt.treasurehunt.AbstractElements;
import com.habii.treasurehunt.treasurehunt.Clue;
import com.habii.treasurehunt.treasurehunt.Player;
import com.habii.treasurehunt.treasurehunt.Post;
import com.habii.treasurehunt.treasurehunt.Task;
import com.habii.treasurehunt.treasurehunt.impl.TreasureHuntImpl;

public class EngineWorld extends TreasureHuntImpl {
	/** Whether actors are bound to stay inside the world */
	private boolean isBounded;
	private GameUI gui;

	public EngineWorld() {
		this(true);
	}

	public EngineWorld(boolean bounded) {
		this.isBounded = bounded;
	}

	/**
	 * Repaints the world.
	 */
	public void updateMap() {
		if (config == null) {
			return;
		}
		((EngineConfig) config).repaintMap(this, gui);
	}

	public void updateEditor(EnginePlayer actor) {
		((EngineConfig) config).repaintEditor(actor, gui);
	}

	public ArrayList<Post> getAllPosts() {
		EList<AbstractElements> elements = this.getElements();
		ArrayList<Post> posts = new ArrayList<Post>();
		for (AbstractElements e : elements) {
			if (e instanceof Post) {
				posts.add((Post) e);
			}
		}
		return posts;
	}

	public ArrayList<EnginePlayer> getAllPlayers() {
		EList<AbstractElements> elements = this.getElements();
		ArrayList<EnginePlayer> players = new ArrayList<EnginePlayer>();
		for (AbstractElements e : elements) {
			if (e instanceof Player) {
				players.add((EnginePlayer) e);
			}
		}
		return players;
	}

	public void setGui(GameUI gui) {
		this.gui = gui;
	}

	public GameUI getGui() {
		return this.gui;
	}

	/**
	 * Test whether this world is bounded.
	 */
	public boolean isBounded() {
		return isBounded;
	}

	public List<Task> getAllTasks() {
		ArrayList<Task> tasks = new ArrayList<Task>();
		for (Post p : getAllPosts()) {
			tasks.add(p.getTask());
		}
		return tasks;
	}

	public List<Clue> getAllClues() {
		ArrayList<Clue> allClues = new ArrayList<Clue>();
		for (Post p : getAllPosts()) {
			allClues.add(p.getClue());
		}
		return allClues;
	}
	
	public Post getPost(Task t){
		for (Post p : getAllPosts()) {
			if(p.getTask().equals(t)){
				return p;
			}
		}
		return null;
	}

	public Post getPost(String postName) {
		for (Post p : getAllPosts()) {
			if(p.getName().equalsIgnoreCase(postName)){
				return p;
			}
		}
		return null;
	}

}
