package com.habii.treasurehunt.framework.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;

import javax.measure.unit.NonSI;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.jscience.geography.coordinates.LatLong;

import com.habii.treasurehunt.framework.EnginePlayer;
import com.habii.treasurehunt.framework.EngineWorld;
import com.habii.treasurehunt.framework.ui.editor.AbstractEditor;
import com.habii.treasurehunt.framework.ui.editor.ConfigEditor;
import com.habii.treasurehunt.framework.ui.editor.PlayerEditor;
import com.habii.treasurehunt.framework.ui.editor.PostEditor;
import com.habii.treasurehunt.framework.ui.map.BaseWaypoint;
import com.habii.treasurehunt.framework.ui.map.PlayerWayPoint;
import com.habii.treasurehunt.framework.ui.map.PostWayPoint;
import com.habii.treasurehunt.framework.util.SampleFileFilter;
import com.habii.treasurehunt.framework.util.State;
import com.habii.treasurehunt.treasurehunt.Clue;
import com.habii.treasurehunt.treasurehunt.Config;
import com.habii.treasurehunt.treasurehunt.Post;
import com.habii.treasurehunt.treasurehunt.Task;
import com.habii.treasurehunt.treasurehunt.TreasurehuntFactory;

public class GameUI extends JFrame implements ActionListener {
	private static final long serialVersionUID = 1L;
	public EngineWorld world;

	private MapPanel mapPanel;
	private CardPanel editors;
	private HashMap<String, AbstractEditor> cardMap;

	private ConfigEditor configEditor;
	private PostEditor postEditor;
	private PlayerEditor playerEditor;

	private JSplitPane splitPane;

	private JToolBar tool;
	private JMenuBar menu;

	private String file;
	private ResourceSet rs;
	private JButton runButton;
	private JButton createPlayerButton;
	private JButton createPostButton;
	private JButton importButton;
	private JButton exportButton;
	private JButton configButton;
	
	private State state;

	public GameUI(String title, int width, int height) {
		setTitle(title);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		menu = createMenuBarAndAddMenus();
		setJMenuBar(menu);

		tool = createToolBarAndAddButtons();
		getContentPane().add(tool, BorderLayout.NORTH);

		editors = new CardPanel();
		cardMap = new HashMap<String, AbstractEditor>();
		mapPanel = new MapPanel(editors, cardMap);

		Dimension dimention = new Dimension(380, 600);

		configEditor = new ConfigEditor(dimention);
		postEditor = new PostEditor(dimention);
		playerEditor = new PlayerEditor(dimention);

		final JPanel jPanel = new JPanel();
		editors.add(jPanel, CardPanel.EMPTY);
		editors.add(configEditor, CardPanel.CONFIG);
		editors.add(postEditor, CardPanel.POST);
		editors.add(playerEditor, CardPanel.PLAYER);

		cardMap.put(CardPanel.EMPTY, null);
		cardMap.put(CardPanel.CONFIG, configEditor);
		cardMap.put(CardPanel.POST, postEditor);
		cardMap.put(CardPanel.PLAYER, playerEditor);

		mapPanel.setMinimumSize(dimention);
		editors.setMinimumSize(dimention);

		mapPanel.setPreferredSize(dimention);

		splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, mapPanel,
				editors);
		splitPane.setBackground(Color.LIGHT_GRAY);
		splitPane.setOneTouchExpandable(true);

		getContentPane().add(splitPane, BorderLayout.CENTER);

		pack();
		setSize(800, 600);
		setVisible(true);
		
		changeState(State.INIT);
	}

	
	private JToolBar createToolBarAndAddButtons() {
		JToolBar tool = new JToolBar("Toolbar", JToolBar.HORIZONTAL);
		runButton = new JButton(new ImageIcon(getClass().getResource("/icons/Play-Green-Button-icon.png")));
		createPostButton = new JButton(new ImageIcon(getClass().getResource("/icons/flag-icon.png")));
		createPlayerButton = new JButton(new ImageIcon(getClass().getResource("/icons/add-user-icon.png")));
		configButton = new JButton(new ImageIcon(getClass().getResource("/icons/Misc-Settings-icon.png")));
		importButton = new JButton(new ImageIcon(getClass().getResource("/icons/import-icon.png")));
		exportButton = new JButton(new ImageIcon(getClass().getResource("/icons/export-icon.png")));

		importButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				action("I");
			}
		});
		runButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				action("R");
			}
		});
		createPlayerButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				action("Player");
			}
		});
		createPostButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				action("Post");
			}
		});
		configButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				action("Config");
			}
		});
		exportButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				action("E");
			}
		});

		tool.add(importButton);
		tool.add(createPostButton);
		tool.add(createPlayerButton);
		tool.add(configButton);
		tool.add(runButton);
		tool.add(exportButton);
		tool.setFloatable(false);

		return tool;
	}

	private JMenuBar createMenuBarAndAddMenus() {
		JMenuBar bar = new JMenuBar();
		createFileMenu(bar);
		createActionMenu(bar);
		createHelpMenu(bar);
		return bar;
	}

	private void createHelpMenu(JMenuBar bar) {
		JMenu menu = new JMenu("Help");
		menu.setMnemonic('H');
		bar.add(menu);
		JMenuItem item;

		menu.add(item = new JMenuItem("Treasure hunt", 'T'));
		item.setAccelerator(KeyStroke.getKeyStroke('T'));
		item.addActionListener(this);

	}

	private void createActionMenu(JMenuBar bar) {
		JMenu menu = new JMenu("Game Actions");
		menu.setMnemonic('G');
		bar.add(menu);
		JMenuItem item;

		menu.add(item = new JMenuItem("Move North", 'W'));
		item.setAccelerator(KeyStroke.getKeyStroke('W'));
		item.addActionListener(this);

		menu.add(item = new JMenuItem("Move South", 'S'));
		item.setAccelerator(KeyStroke.getKeyStroke('S'));
		item.addActionListener(this);

		menu.add(item = new JMenuItem("Move West", 'A'));
		item.setAccelerator(KeyStroke.getKeyStroke('A'));
		item.addActionListener(this);

		menu.add(item = new JMenuItem("Move East", 'D'));
		item.setAccelerator(KeyStroke.getKeyStroke('D'));
		item.addActionListener(this);
	}

	private void createFileMenu(JMenuBar bar) {
		JMenu menu = new JMenu("File");
		menu.setMnemonic('F');
		bar.add(menu);
		JMenuItem item;

		menu.add(item = new JMenuItem("Close", 'X'));
		item.setAccelerator(KeyStroke.getKeyStroke('X'));
		item.addActionListener(this);
		menu.add(new JSeparator()); // SEPARATOR
		menu.add(item = new JMenuItem("Import", 'I'));
		item.setAccelerator(KeyStroke.getKeyStroke('I'));
		item.addActionListener(this);

		menu.add(item = new JMenuItem("Export", 'E'));
		item.setAccelerator(KeyStroke.getKeyStroke('E'));
		item.addActionListener(this);
		menu.add(new JSeparator()); // SEPARATOR
		menu.add(item = new JMenuItem("Exit", 'Q'));
		item.setAccelerator(KeyStroke.getKeyStroke('Q'));
		item.addActionListener(this);

	}

	private void action(String cmd) {
		if (cmd.equalsIgnoreCase("R")) {
			run();
			changeState(State.RUN);
		} else if (cmd.equalsIgnoreCase("Player")) {
			createNewPlayer();
			changeState(State.EDIT);
		} else if (cmd.equalsIgnoreCase("Post")) {
			createNewPost();
			changeState(State.EDIT);
		} else if (cmd.equalsIgnoreCase("Config")) {
			showOrCreateConfig();
			changeState(State.EDIT);
		} else if (cmd.equalsIgnoreCase("I")) {
			load();
			changeState(State.IMPORT);
		} else if (cmd.equalsIgnoreCase("E")) {
			export();
			changeState(State.EXPORT);
		} else {

		}
	}

	private void run() {
		if(world==null){
			System.out.println("Nothing to run.");
		} else {
			for (EnginePlayer p : world.getAllPlayers()) {
				p.start();
			}
		}
		
	}

	@Override
	public void actionPerformed(ActionEvent evt) {
		String cmd = evt.getActionCommand();
		System.out.println("You chose " + cmd);
		if (cmd.equalsIgnoreCase("Import")) {
			load();
		} else if (cmd.equalsIgnoreCase("Export")) {
			export();
		}
	}

	private void export() {
		if (world == null) {
			return;
		}
		for (Resource s : rs.getResources()) {
			if (s.getURI().fileExtension().equals("th")) {
				JFileChooser chooser = new JFileChooser();
				SampleFileFilter filter = new SampleFileFilter();
				filter.addType("th");
				filter.setDescription("TH (Treasurehunt) files only");
				chooser.setFileFilter(filter);
				int returnVal = chooser.showSaveDialog(this);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					String path = chooser.getSelectedFile().getPath();
					s.setURI(URI.createFileURI(path));
					try {
						s.save(null);
					} catch (IOException e) {
						e.printStackTrace();
					}
					System.out.println("Saved into: " + path);
					return;
				}
			}
		}
		world.updateMap();
	}

	private void load() {
		clean();
		world = loadGameInstance();
		setGUI();
	}

	private void setGUI() {
		world.setGui(this);
		mapPanel.setWaypoints(new HashSet<BaseWaypoint>());
		world.updateMap();
	}

	private void clean() {
		world = null;
	}

	private EngineWorld loadGameInstance() {
		JFileChooser chooser = new JFileChooser();
		SampleFileFilter filter = new SampleFileFilter();
		filter.addType("th");
		filter.setDescription("TH (Treasurehunt) files only");
		chooser.setFileFilter(filter);
		int returnVal = chooser.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			this.file = chooser.getSelectedFile().getPath();
			rs = new ResourceSetImpl();
			Resource resource = rs.getResource(URI.createURI(file), true);
			EngineWorld world = (EngineWorld) resource.getContents().get(0);
			return world;
		} else {
			//
		}
		return null;
	}
	
	
	
	private void createWorld() {
		rs = new ResourceSetImpl();
		
//		Resource resource = rs.getResource(URI.createURI("resources/init/empty.th"), true);
		URI createFileURI = URI.createURI(getClass().getResource("/init/empty.th").toString());
		Resource resource = rs.getResource(createFileURI, true);
		
		createPlayerButton = new JButton(new ImageIcon());
		world = (EngineWorld) resource.getContents().get(0);
		setGUI();
	}
	

	private void showOrCreateConfig() {
		if(world==null){
			createWorld();
		}
		
		Config c = world.getConfig();
		editors.showConfigPanel();
		ConfigEditor configEditor = (ConfigEditor) cardMap.get(CardPanel.CONFIG);
		configEditor.update(c);
	}
	
	private void createNewPlayer() {
		if(world==null){
			createWorld();
		}
		
		if(world.getAllPosts().isEmpty()){
			System.out.println("Du m frst lage en post.");
			return;
		}
		
		int id = calculateUniqueID();
		
		EnginePlayer player = (EnginePlayer) TreasurehuntFactory.eINSTANCE.createPlayer();
		player.setName("Pl"+id);
		player.setFullname("PlayerName"+id);
		
		player.setInitialClue(world.getAllPosts().get(0).getClue());
		
		double d = calculateDropPoint();
		
		player.setCurrentPosition(LatLong.valueOf(MapPanel.DEFUALT_LATITUDE,
				MapPanel.DEFAULT_LONGITUDE - d, NonSI.DEGREE_ANGLE));
		
		world.getElements().add(player);
		player.setWorld(world);
		
		mapPanel.addWayPoint(new PlayerWayPoint(player));
		world.updateMap();
	}

	private void createNewPost() {
		if(world==null){
			createWorld();
		}
		
		int id = calculateUniqueID();
		Post p = TreasurehuntFactory.eINSTANCE.createPost();
		p.setName("P"+id);
		p.setTitle("Post"+id);
		
		Task t = TreasurehuntFactory.eINSTANCE.createTask();
		t.setName("t"+id);
		t.setAnswer("42");
		t.setQuestion("What is the ultimate answer");
		p.setTask(t);
		
		double d = calculateDropPoint();
		p.setPosition(LatLong.valueOf(MapPanel.DEFUALT_LATITUDE,
				MapPanel.DEFAULT_LONGITUDE + d, NonSI.DEGREE_ANGLE));
		
		Clue c = TreasurehuntFactory.eINSTANCE.createClue();
		c.setName("c"+id);
		c.setText("Clue for finding next post");
		p.setClue(c);
		
		world.getElements().add(p);
		p.setWorld(world);
		
		mapPanel.addWayPoint(new PostWayPoint(p));
		world.updateMap();
	}
	

	private int calculateUniqueID(){
		if(world==null){
			return 0;
		} else {
			return world.getElements().size();
		}
	}
	
	private double calculateDropPoint() {
		return calculateUniqueID()*0.001;
	}

	public MapPanel getMap() {
		return mapPanel;
	}

	public void updateAll(EnginePlayer actor) {
		mapPanel.repaint();
		playerEditor.update(actor);
	}
	
	private void changeState(State state){
		this.setState(state);
		switch (state) {
		case IMPORT:
			importButton.setEnabled(false);
			
			createPostButton.setEnabled(true);
			createPlayerButton.setEnabled(true);
			configButton.setEnabled(true);
			runButton.setEnabled(true);
			exportButton.setEnabled(true);
			break;
		case EDIT:
			importButton.setEnabled(false);
			
			createPostButton.setEnabled(true);
			createPlayerButton.setEnabled(true);
			configButton.setEnabled(true);
			runButton.setEnabled(true);
			exportButton.setEnabled(true);
			break;
		case RUN:
			importButton.setEnabled(false);
			createPostButton.setEnabled(false);
			createPlayerButton.setEnabled(false);
			configButton.setEnabled(false);
			
			exportButton.setEnabled(true);
			break;
		case EXPORT:
			importButton.setEnabled(false);
			createPostButton.setEnabled(false);
			createPlayerButton.setEnabled(false);
			configButton.setEnabled(false);
			runButton.setEnabled(false);
			exportButton.setEnabled(false);
			break;
		case INIT:
			importButton.setEnabled(true);
			createPostButton.setEnabled(true);
			createPlayerButton.setEnabled(true);
			configButton.setEnabled(true);
			
			runButton.setEnabled(false);
			exportButton.setEnabled(false);
			break;
		default:
			break;
		}
	}


	public int getState() {
		return state.getValue();
	}


	public void setState(State state) {
		this.state = state;
	}
}
