package com.habii.treasurehunt.framework.ui;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Point2D;
import java.util.HashMap;
import java.util.Set;

import javax.measure.unit.NonSI;
import javax.swing.JOptionPane;
import javax.swing.ToolTipManager;
import javax.swing.event.EventListenerList;

import org.jdesktop.swingx.JXMapKit;
import org.jdesktop.swingx.JXMapViewer;
import org.jdesktop.swingx.mapviewer.GeoPosition;
import org.jdesktop.swingx.mapviewer.Waypoint;
import org.jdesktop.swingx.mapviewer.WaypointPainter;
import org.jscience.geography.coordinates.LatLong;

import com.habii.treasurehunt.framework.EnginePlayer;
import com.habii.treasurehunt.framework.EngineWorld;
import com.habii.treasurehunt.framework.ui.editor.AbstractEditor;
import com.habii.treasurehunt.framework.ui.editor.PlayerEditor;
import com.habii.treasurehunt.framework.ui.editor.PostEditor;
import com.habii.treasurehunt.framework.ui.map.BaseWaypoint;
import com.habii.treasurehunt.framework.ui.map.BaseWaypointPainter;
import com.habii.treasurehunt.framework.ui.map.PlayerWayPoint;
import com.habii.treasurehunt.framework.ui.map.PostWayPoint;
import com.habii.treasurehunt.treasurehunt.Post;
import com.habii.treasurehunt.treasurehunt.Task;

public class MapPanel extends JXMapKit{
	private static final long serialVersionUID = 1787118143488776032L;
	public final static double DEFUALT_LATITUDE = 63.430476;
	public final static double DEFAULT_LONGITUDE = 10.39255;
	public final int DEFAULT_ZOOM = 3;
	
	private CardPanel editors;
	private HashMap<String, AbstractEditor> cardMap;
	
	private EnginePlayer selectedPlayer;
	private Post selectedPost;
	
	protected EventListenerList _waypointEventListeners = new EventListenerList();
	
	protected Set<BaseWaypoint> waypoints;
	private JXMapViewer map;

	public MapPanel(CardPanel editors, HashMap<String, AbstractEditor> cardMap){
		init(DEFUALT_LATITUDE, DEFAULT_LONGITUDE);
		this.editors = editors;
		this.cardMap = cardMap;
	}
	
	private void init(double latitude, double longitude){
		setDefaultProvider(DefaultProviders.OpenStreetMaps);
		setCenterPosition(new GeoPosition(latitude, longitude));
		setZoom(DEFAULT_ZOOM);
		setMiniMapVisible(false);
		setZoomButtonsVisible(true);
		setZoomSliderVisible(false);
		
		ToolTipManager.sharedInstance().setInitialDelay(0);
		
		map = getMainMap();
		
    	map.setZoomEnabled(false);
    	map.setPanEnabled(true);
		map.addMouseListener(new MouseListener(){
			@Override
			public void mouseClicked(MouseEvent event) {
				Waypoint waypoint = getWaypoint(event.getPoint());
                if (waypoint != null){
                	if(waypoint instanceof PlayerWayPoint){
                		playerWayPointUpdate(waypoint);
                	} else if (waypoint instanceof PostWayPoint){
                		postWayPointUpdate(waypoint);
                	}
                } else {
            		PostEditor editor = (PostEditor) cardMap.get(CardPanel.POST);
            		Post post = editor.getSelectedPost();
                	
                	if(post!=null){
                		LatLong coordinates = getCoordinates(event.getPoint());
                		post.setPosition(coordinates);
                		EngineWorld world = (EngineWorld) post.getWorld();
						world.updateMap();
                		editor.update(post);
                	}
                }
			}
			
			@Override
			public void mouseEntered(MouseEvent event) {}
			@Override
			public void mouseExited(MouseEvent event) {}
			@Override
			public void mousePressed(MouseEvent event) {}
			@Override
			public void mouseReleased(MouseEvent event) {}
	    });
		
		map.addKeyListener(new KeyListener() {
			@Override
			public void keyTyped(KeyEvent e) {
				char keyChar = e.getKeyChar();
				if(keyChar=='w'){
					doMove("W");
				} else if(keyChar=='s'){
					doMove("S");
				} else if(keyChar=='a'){
					doMove("A");
				} else if(keyChar=='d'){
					doMove("D");
				} else {
				}
			}
			@Override
			public void keyReleased(KeyEvent e) { }
			@Override
			public void keyPressed(KeyEvent e) { }
		});
		
	}
	
	public LatLong getCoordinates(Point mousePoint){
		GeoPosition centerPosition = getMainMap().getCenterPosition();
		double latitude = centerPosition.getLatitude();
		double longitude = centerPosition.getLongitude();
		return LatLong.valueOf(latitude, longitude, NonSI.DEGREE_ANGLE);
	}
	
	private void playerWayPointUpdate(Waypoint waypoint) {
		PlayerWayPoint playerWayPoint = (PlayerWayPoint) waypoint;
		selectedPlayer = (EnginePlayer) playerWayPoint.getPlayer();
		editors.showPlayerPanel();
		PlayerEditor playerEditor = (PlayerEditor) cardMap.get(CardPanel.PLAYER);
		playerEditor.update(selectedPlayer);
		
	}
	
	private void postWayPointUpdate(Waypoint waypoint) {
		PostWayPoint postWayPoint = (PostWayPoint) waypoint;
		selectedPost = postWayPoint.getPost();
		
		editors.showPostPanel();
		PostEditor postEditor = (PostEditor) cardMap.get(CardPanel.POST);
		postEditor.update(selectedPost);
	}
	
	@SuppressWarnings("all")
	public void setWaypoints(Set<BaseWaypoint> waypoints){
		this.waypoints = waypoints;
		WaypointPainter painter = new BaseWaypointPainter();
	    painter.setWaypoints(waypoints);
	    getMainMap().setOverlayPainter(painter);
	}
	
	public Waypoint getWaypoint(Point mousePoint){
		if (waypoints == null){
			return null;
		}
		
		JXMapViewer map = getMainMap();
        
		Rectangle bounds = map.getViewportBounds();
		for(BaseWaypoint waypoint : waypoints){
			Point2D point = map.getTileFactory().geoToPixel(waypoint.getPosition(), map.getZoom());
			
			int x = (int)(point.getX() - bounds.getX());
            int y = (int)(point.getY() - bounds.getY());
            int s = waypoint.getSize();
            
            if (new Rectangle(x - s, y - s, s * 2, s * 2).contains(mousePoint)){
            	return waypoint;
            }
		}
		
		return null;
	}
	
	public void addWayPoint(BaseWaypoint waypoint){
		waypoints.add(waypoint);
	}
	
	public void cleanWayPoints(){
		waypoints.clear();
	}

	public EnginePlayer getSelectedPlayer() {
		return selectedPlayer;
	}
	
	public Post getSelectedPost() {
		return selectedPost;
	}
	
	private void doMove(String cmd) {
		if(selectedPlayer==null){
			return;
		}
		selectedPlayer.doMove(cmd);
		Post post = selectedPlayer.calculateDistanceToClosestPost();
		if(post!=null){
			Task t = post.getTask();
			String answer = JOptionPane.showInputDialog(t.getQuestion()+"?");
			if(t.getAnswer().equalsIgnoreCase(answer)){
				if(selectedPlayer.calculateStopWatch()>0){
					selectedPlayer.solved(t);
					t.getSolvedBy().add(selectedPlayer);
				}
			}
		}
		
		EngineWorld world = (EngineWorld) selectedPlayer.getWorld();
		world.updateMap();
		world.updateEditor(selectedPlayer);
	}
}
