package com.habii.treasurehunt.framework.ui.editor;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;

import javax.imageio.ImageIO;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import com.habii.treasurehunt.framework.EngineWorld;
import com.habii.treasurehunt.treasurehunt.Config;
import com.habii.treasurehunt.treasurehunt.Level;
import com.habii.treasurehunt.treasurehunt.Order;
import com.habii.treasurehunt.treasurehunt.Post;
import com.habii.treasurehunt.treasurehunt.Temporal;
import com.habii.treasurehunt.treasurehunt.impl.TreasurehuntFactoryImpl;

public class ConfigEditor extends AbstractEditor{
	private static final long serialVersionUID = 838777207262673456L;
	
	private ImagePanel imagePanel;
	private Config config = null;
	
	private JComboBox goalJCB;
	private JComboBox mobilityJCB;
	private JComboBox temporalJCB;
	private JComboBox socialJCB;
	
	private JTextArea orderingFields;
	private JTextField days;
	private JTextField hours;
	private JTextField minutes;
	private JTextField seconds;
	private JTextField postsFields;
	private JTextField teams;
	
	public ConfigEditor(Dimension dimention) {
		super(dimention);
		initializeFields();
		addListeners();
		addFields();
	}
	
	private void initializeFields() {
		imagePanel = new ImagePanel();
		Dimension axisSize = imagePanel.getPreferredSize();
        axisSize.height = 230;
        imagePanel.setPreferredSize(axisSize);
		
		days = new JTextField();
		days.addKeyListener(new DeadlineKeyListener("d"));
        hours = new JTextField();
        hours.addKeyListener(new DeadlineKeyListener("h"));
        minutes = new JTextField();
        minutes.addKeyListener(new DeadlineKeyListener("m"));
        seconds = new JTextField();
        seconds.addKeyListener(new DeadlineKeyListener("s"));
        
        orderingFields = new JTextArea();
        orderingFields.setRows(2);
        
        orderingFields.addKeyListener(new KeyListener() {
			@Override
			public void keyTyped(KeyEvent e) {}
			@Override
			public void keyReleased(KeyEvent e) { }
			@Override
			public void keyPressed(KeyEvent e) {
				if(KeyEvent.VK_ENTER==e.getKeyCode()){
					ArrayList<Order> validOrders = getValidOrders(orderingFields.getText());
					config.getMobilityAxis().getOrdering().clear();
					config.getMobilityAxis().getOrdering().addAll(validOrders);
				}
			}
		});
        
        postsFields = new JTextField();
        postsFields.addKeyListener(new KeyListener() {
			@Override
			public void keyTyped(KeyEvent e) {}
			@Override
			public void keyReleased(KeyEvent e) { }
			@Override
			public void keyPressed(KeyEvent e) {
				if(KeyEvent.VK_ENTER==e.getKeyCode()){
					ArrayList<Post> validPosts = getValidPosts(postsFields.getText());
					config.getTemporalAxis().getPosts().clear();
					config.getTemporalAxis().getPosts().addAll(validPosts);
				}
			}
		});
        teams = new JTextField();
        
        goalJCB = createJCB();
        goalJCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Level l = (Level) goalJCB.getSelectedItem();
				config.setGoal(l);
			}
		});
        
        mobilityJCB = createJCB();
        mobilityJCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Level l = (Level) mobilityJCB.getSelectedItem();
				config.getMobilityAxis().setLevel(l);
			}
		});
        temporalJCB = createJCB();
        temporalJCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Level l = (Level) temporalJCB.getSelectedItem();
				config.getTemporalAxis().setLevel(l);
			}
		});
        
        socialJCB = createJCB();
        socialJCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				Level l = (Level) socialJCB.getSelectedItem();
				config.getSocialAxis().setLevel(l);
			}
		});
		
	}
	
	class DeadlineKeyListener implements KeyListener{
		private String field;
		public DeadlineKeyListener(String field) {
			super();
			this.field = field;
		}
		@Override
		public void keyPressed(KeyEvent e) {
			if(field.endsWith("d")){
				String text = days.getText().trim();
				if(text==""){
					return;
				}
				int d = Integer.parseInt(text);
				config.getTemporalAxis().getDeadline().setDays(d);
			} else if(field.endsWith("h")){
				String text = hours.getText().trim();
				if(text==""){
					return;
				}
				int d = Integer.parseInt(text);
				config.getTemporalAxis().getDeadline().setHours(d);
			} else if(field.endsWith("m")){
				String text = minutes.getText().trim();
				if(text==""){
					return;
				}
				int d = Integer.parseInt(text);
				config.getTemporalAxis().getDeadline().setMinutes(d);
			} else if(field.endsWith("s")){
				String text = seconds.getText().trim();
				if(text==""){
					return;
				}
				int d = Integer.parseInt(text);
				config.getTemporalAxis().getDeadline().setSeconds(d);
			}
		}
		@Override
		public void keyReleased(KeyEvent e) { }

		@Override
		public void keyTyped(KeyEvent e) {}
	}
	
	private ArrayList<Order> getValidOrders(String nyOrderings) {
		String[] orders = nyOrderings.split(",");
		
		ArrayList<Order> nyOrders = new ArrayList<Order>();
		for (int i = 0; i < orders.length; i++) {
			String[] posts = orders[i].split("->");
			EngineWorld world = (EngineWorld) config.getWorld();
			Post p1 = world.getPost(posts[0].trim());
			Post p2 = world.getPost(posts[1].trim());
			
			if(p1==null||p2==null){
				break;
			}
			
			Order o = TreasurehuntFactoryImpl.eINSTANCE.createOrder();
			o.setP1(p1);
			o.setP2(p2);
			nyOrders.add(o);
		}
		return nyOrders;
	}
	
	private ArrayList<Post> getValidPosts(String textPosts) {
		String[] posts = textPosts.split(",");
		
		ArrayList<Post> nyPosts = new ArrayList<Post>();
		for (int i = 0; i < posts.length; i++) {
			Post p = ((EngineWorld) config.getWorld()).getPost(posts[i]);
			if(p==null){
				break;
			}
			nyPosts.add(p);
		}
		return nyPosts;
	}
	
	
	private void addFields() {
		formUtility.addLastField(imagePanel, form);
		formUtility.addLastField(new JSeparator(),form);
		
		formUtility.addLabel("Goal: ", form);
        formUtility.addLastField(goalJCB, form);
        formUtility.addLastField(new JSeparator(),form);
        
        formUtility.addLabel("Mobility: ", form);
        formUtility.addLastField(mobilityJCB, form);
        formUtility.addLabel("Ordering", form);
        
		formUtility.addLastField(orderingFields, form);
        formUtility.addLastField(new JSeparator(),form);
        
        formUtility.addLabel("Temporal: ", form);
        formUtility.addLastField(temporalJCB, form);
        formUtility.addLabel("Deadline: ", form);
        formUtility.addMiddleField(new JLabel(), form);

        JPanel stateZip = new JPanel();
        stateZip.setLayout(new GridBagLayout());
        
        formUtility.addLabel(" D: ", stateZip);
        Dimension daysSize = days.getPreferredSize();
        daysSize.width = 30;
        days.setPreferredSize(daysSize);
        formUtility.addLabel(days, stateZip);
        
        formUtility.addLabel(" H: ", stateZip);
        Dimension hoursSize = hours.getPreferredSize();
        hoursSize.width = 30;
        hours.setPreferredSize(hoursSize);
        formUtility.addLabel(hours, stateZip);
        
        formUtility.addLabel(" M: ", stateZip);
        Dimension minutesSize = minutes.getPreferredSize();
        minutesSize.width = 30;
        minutes.setPreferredSize(minutesSize);
        formUtility.addLabel(minutes, stateZip);
        
        formUtility.addLabel(" S: ", stateZip);
        Dimension secondsSize = seconds.getPreferredSize();
        secondsSize.width = 30;
        seconds.setPreferredSize(secondsSize);
        formUtility.addLabel(seconds, stateZip);

        formUtility.addLabel(stateZip, form);
        formUtility.addLastField(new JPanel(), form);
        
        formUtility.addLabel("Posts: ", form);
        formUtility.addLastField(postsFields, form);
        formUtility.addLastField(new JSeparator(),form);
        
        formUtility.addLabel("Social: ", form);
        formUtility.addLastField(socialJCB, form);
        formUtility.addLabel("Teams: ", form);
        formUtility.addLastField(teams, form);
		
	}
	
	private JComboBox createJCB() {
		JComboBox jcb = new JComboBox();
        jcb.addItem(Level.LOW);
        jcb.addItem(Level.MEDIUM);
        jcb.addItem(Level.HIGH);
		return jcb;
	}
	
	private void addListeners() {
		goalJCB.addActionListener(new GoalActionListener());
        mobilityJCB.addActionListener(new MobilityActionListener());
        temporalJCB.addActionListener(new TemporalActionListener());
        socialJCB.addActionListener(new SocialActionListener());
	}
	
	public void update(Object actor){
		if(actor==null){
			return;
		}
		
		if (actor instanceof Config) {
			config = (Config) actor;
			showInfo(config);
		} 
	}
	
	private void showInfo(Config config) {
		goalJCB.setSelectedItem(config.getGoal());
		mobilityJCB.setSelectedItem(config.getMobilityAxis().getLevel());
		String orders = "";
		for (Order o : config.getMobilityAxis().getOrdering()) {
			orders+=o.getP1().getName().toString()+"->"+o.getP2().getName().toString()+",";
		}
		
		if(orders.length()>5){
			int l = orders.length()-1;
			orderingFields.setText(orders.substring(0, l));
		}
		
		Temporal t = config.getTemporalAxis();
		temporalJCB.setSelectedItem(t.getLevel());
		
		if(t.getDeadline()!=null){
			days.setText(t.getDeadline().getDays()+"");
			hours.setText(t.getDeadline().getHours()+"");
			minutes.setText(t.getDeadline().getMinutes()+"");
			seconds.setText(t.getDeadline().getSeconds()+"");
		}
		
		String posts = "";
		for (Post p : t.getPosts()) {
			posts+=p.getName().toString()+",";
		}
		
		if(posts.length()>1){
			int lp = posts.length()-1;
			postsFields.setText(posts.substring(0, lp));
		}
		socialJCB.setSelectedItem(config.getSocialAxis().getLevel());
	}
	
	class GoalActionListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			Level gLevel = (Level) goalJCB.getSelectedItem();
			switch (gLevel) {
			case HIGH:
				break;
			case MEDIUM:
				break;
			case LOW:
				break;
			default:
				break;
			}
			imagePanel.changed(gLevel, 'g');			
		}
	}
	
	
	class MobilityActionListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			Level mLevel = (Level) mobilityJCB.getSelectedItem();
			
			switch (mLevel) {
			case HIGH:
				orderingFields.setEnabled(true);
				break;
			case MEDIUM:
				orderingFields.setEnabled(true);
				break;
			case LOW:
				orderingFields.setEnabled(false);
				break;
			default:
				
				break;
			}
			imagePanel.changed(mLevel, 'm');		
		}
	}
	
	class TemporalActionListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			Level tLevel = (Level) temporalJCB.getSelectedItem();
			
			switch (tLevel) {
			case HIGH:
				postsFields.setEnabled(false);
				days.setEnabled(true);
				hours.setEnabled(true);
				minutes.setEnabled(true);
				seconds.setEnabled(true);
				break;
			case MEDIUM:
				postsFields.setEnabled(true);
				days.setEnabled(true);
				hours.setEnabled(true);
				minutes.setEnabled(true);
				seconds.setEnabled(true);
				break;
			case LOW:
				postsFields.setEnabled(false);
				days.setEnabled(false);
				hours.setEnabled(false);
				minutes.setEnabled(false);
				seconds.setEnabled(false);
				break;

			default:
				break;
			}
			imagePanel.changed(tLevel, 't');			
		}
	}
	
	class SocialActionListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent e) {
			Level slevel = (Level) socialJCB.getSelectedItem();
			switch (slevel) {
			case HIGH:
				teams.setEnabled(true);
				break;
			case MEDIUM:
				teams.setEnabled(false);
				break;
			case LOW:
				teams.setEnabled(false);
				break;
			default:
				break;
			}
			imagePanel.changed(slevel, 's');			
		}
	}

	public class ImagePanel extends JPanel{
		private static final long serialVersionUID = 1L;
		private int goal = 1;
		private int mobility = 1;
		private int temporal = 1;
		private int social = 1;
		
		private String url;
		
		private BufferedImage image;
		
	    public ImagePanel() {
	    	url = "http://chart.apis.google.com/chart?chxl=0:|G|S|T|M|1:||LOW|MEDIUM|HIGH&chxr=0,1,4|1,0,5&chxt=x,y&chs=230x230&cht=r&chco=FF00009B&chds=0,3&chd=t:"+goal+","+social+","+temporal+","+mobility+","+goal+"&chls=2,4,0&chm=B,00800072,0,0,0";
			try {
				image = ImageIO.read(new URL(url));
			} catch (IOException ex) {
				System.out.println("Klarte ikke  laste ned bildet");
			}
	    }
	    
	    public void changed(Level level, char field) {
	    	switch (field) {
			case 'g':
				goal = level.getValue()+1;
				break;
			case 'm':
				mobility = level.getValue()+1;
				break;
			case 't':
				temporal = level.getValue()+1;
				break;
			case 's':
				social = level.getValue()+1;
				break;
			default:
				break;
			}
	    	repaint();
	    }

	    @Override
	    public void paintComponent(Graphics g) {
	    	url = "http://chart.apis.google.com/chart?chxl=0:|G|S|T|M|1:||LOW|MEDIUM|HIGH&chxr=0,1,4|1,0,5&chxt=x,y&chs=230x230&cht=r&chco=FF00009B&chds=0,3&chd=t:"+goal+","+social+","+temporal+","+mobility+","+goal+"&chls=2,4,0&chm=B,00800072,0,0,0";
	    	try {
				image = ImageIO.read(new URL(url));
			} catch (IOException ex) {
				System.out.println("Klarte ikke  laste ned bildet");
			}
	    	g.drawImage(image, 0, 0, null);
	    }
	}
}
