package com.habii.treasurehunt.framework.ui.editor;

import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.measure.unit.NonSI;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.habii.treasurehunt.framework.EnginePlayer;
import com.habii.treasurehunt.framework.EngineWorld;
import com.habii.treasurehunt.treasurehunt.Clue;

public class PlayerEditor extends AbstractEditor{
	private static final long serialVersionUID = 1L;
	
	private JTextField fullName;
	private JComboBox initialClueJCB;
	private JTextField posLat;
	private JTextField posLong;
	
	
	private JLabel playerName;
	private JLabel timeSpent;
	private JLabel stopWatch;
	private JLabel points;
	private JLabel distanceToClosestPost;

	private JButton delete;
	
	private EnginePlayer selectedPlayer = null;

	public PlayerEditor(Dimension dimention) {
		super(dimention);
		initializeFields();
		addFields();
	}
	
	private void initializeFields() {
		playerName = new JLabel("");
		timeSpent = new JLabel("");		
		stopWatch = new JLabel("");		
		points = new JLabel(""+0);
		distanceToClosestPost = new JLabel("COLD");		
		
		delete = new JButton("Delete");
		fullName= new JTextField();
		initialClueJCB = new JComboBox();
		posLat = new JTextField();
		posLong = new JTextField();
		
		
		fullName.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				selectedPlayer.setFullname(fullName.getText());
			}
		});
		
		initialClueJCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				Clue choosenInitialClue = (Clue) initialClueJCB.getSelectedItem();
				selectedPlayer.setInitialClue(choosenInitialClue);
			}
		});
		
		delete.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				EngineWorld world = (EngineWorld) selectedPlayer.getWorld();
				world.getElements().remove(selectedPlayer);
				selectedPlayer.setWorld(null);
				world.updateMap();
			}
		});
	}

	private void addFields() {
		formUtility.addLabel("Name: ", form);
        formUtility.addLastField(playerName, form);
        
		formUtility.addLabel("Title: ", form);
        formUtility.addLastField(fullName, form);
        
        formUtility.addLabel("initClue: ", form);
        formUtility.addLastField(initialClueJCB, form);
        
        addPosition(90);
        
        formUtility.addLabel("Time spent: ", form);
        formUtility.addLastField(timeSpent, form);
        
        formUtility.addLabel("Deadline: ", form);
        formUtility.addLastField(stopWatch, form);
        
        formUtility.addLabel("Points: ", form);
        formUtility.addLastField(points, form);
        
        formUtility.addLabel("Close?: ", form);
        formUtility.addLastField(distanceToClosestPost, form);
        
        formUtility.addLabel("Delete: ", form);
        formUtility.addLastField(delete, form);
	}
	
	
	private void addPosition(int width) {
		formUtility.addLabel("Position: ", form);
		JPanel position = new JPanel();
        position.setLayout(new GridBagLayout());
        
        Dimension size = posLat.getPreferredSize();
        size.width = width;
        
        formUtility.addLabel("Lat", position);
        posLat.setPreferredSize(size);
        formUtility.addLabel(posLat, position);
        
        formUtility.addLabel("Long", position);
        posLong.setPreferredSize(size);
        formUtility.addLabel(posLong, position);
        
        formUtility.addLastField(position, form);
	}
	
	public void update(Object actor){
		if(actor==null){
			return;
		}
		if (actor instanceof EnginePlayer) {
			selectedPlayer = (EnginePlayer) actor;
			showInfo(selectedPlayer);
		} else {
			// do nothing
		}
		
	}
	
	public void showInfo(EnginePlayer selectedPlayer){
		fullName.setText(selectedPlayer.getFullname());
		EngineWorld world = (EngineWorld) selectedPlayer.getWorld();
		initialClueJCB.removeAllItems();
		for (Clue c : world.getAllClues()) {
			initialClueJCB.addItem(c);
		}
		initialClueJCB.setSelectedItem(selectedPlayer.getInitialClue());
		
		double latValue = selectedPlayer.getCurrentPosition().latitudeValue(NonSI.DEGREE_ANGLE);
		double longValue = selectedPlayer.getCurrentPosition().longitudeValue(NonSI.DEGREE_ANGLE);
		posLat.setText(latValue+"");
		posLong.setText(longValue+"");
		
		playerName.setText(selectedPlayer.getName());
		
		
		points.setText(selectedPlayer.caculatePoints()+"");
		distanceToClosestPost.setText(""+selectedPlayer.getDistanceToClosestPost());
		
		if(selectedPlayer.getStart()==-1){
			timeSpent.setText("Game not started yet");
			stopWatch.setText("Game not started yet");
		} else {
			long miliseconds = selectedPlayer.calculateTotalTime();
			timeSpent.setText(selectedPlayer.timeAgo(miliseconds));
			
			
			
			long milisecondsStopWatch = selectedPlayer.calculateStopWatch();
			if(milisecondsStopWatch>0){
				stopWatch.setText(selectedPlayer.timeAgo(milisecondsStopWatch));				
			} else {
				stopWatch.setText("timeout!");
			}
		}
	}
}
