package com.habii.treasurehunt.framework.ui.editor;

import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.measure.unit.NonSI;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.habii.treasurehunt.framework.EngineWorld;
import com.habii.treasurehunt.treasurehunt.Post;

public class PostEditor extends AbstractEditor{
	private static final long serialVersionUID = 1L;
	
	private JTextField title;
	private JTextField clueForNextPost;
	private JTextField posLat;
	private JTextField posLong;
	private JTextField tQ;
	private JTextField tA;
	
	private JButton delete;
	
	private Post selectedPost = null;
	
	private JLabel postName;

	public PostEditor(Dimension dimention) {
		super(dimention);
		initializeFields();
		addFields();
	}
	
	private void initializeFields() {
		postName = new JLabel("");
		title= new JTextField();
		title.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent ae) {
				if(selectedPost!=null){
					selectedPost.setTitle(title.getText());
				}
			}
		});
		clueForNextPost = new JTextField();
		clueForNextPost.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent ae) {
				if(selectedPost!=null){
					selectedPost.getClue().setText(clueForNextPost.getText());
				}
			}
		});
		
		posLat = new JTextField();
		posLong = new JTextField();
		
		tQ = new JTextField();
		tQ.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent ae) {
				if(selectedPost!=null){
					selectedPost.getTask().setQuestion(tQ.getText());
				}
			}
		});
		
		
		tA = new JTextField();
		tA.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent ae) {
				if(selectedPost!=null){
					selectedPost.getTask().setAnswer(tA.getText());
				}
			}
		});
		
		delete = new JButton("Delete");
		delete.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				EngineWorld world = (EngineWorld) selectedPost.getWorld();
				world.getElements().remove(selectedPost);
				selectedPost.setWorld(null);
				world.updateMap();
			}
		});
	}
	

	private void addFields() {
		formUtility.addLabel("Name: ", form);
        formUtility.addLastField(postName, form);
        
		formUtility.addLabel("Title: ", form);
        formUtility.addLastField(title, form);
        
        formUtility.addLabel("Clue: ", form);
        formUtility.addLastField(clueForNextPost, form);
        
        addTask(90);
        addPosition(90);
        
        formUtility.addLabel("Delete: ", form);
        formUtility.addLastField(delete, form);
        
	}
	
	private void addTask(int width) {
		formUtility.addLabel("Task: ", form);
		JPanel task = new JPanel();
        task.setLayout(new GridBagLayout());
        
        Dimension size = tQ.getPreferredSize();
        size.width = width;
        
        formUtility.addLabel("Q: ", task);
        tQ.setPreferredSize(size);
        formUtility.addLabel(tQ, task);
        formUtility.addLabel("A", task);
        tA.setPreferredSize(size);
        formUtility.addLabel(tA, task);
        
        formUtility.addLastField(task, form);
	}

	private void addPosition(int width) {
		formUtility.addLabel("Position: ", form);
		JPanel position = new JPanel();
        position.setLayout(new GridBagLayout());
        
        Dimension size = posLat.getPreferredSize();
        size.width = width;
        
        formUtility.addLabel("Lat", position);
        posLat.setPreferredSize(size);
        formUtility.addLabel(posLat, position);
        
        formUtility.addLabel("Long", position);
        posLong.setPreferredSize(size);
        formUtility.addLabel(posLong, position);
        
        formUtility.addLastField(position, form);
	}
	
	public void update(Object actor){
		if(actor==null){
			return;
		}
		if (actor instanceof Post) {
			selectedPost = (Post) actor;
			showInfo(selectedPost);
		}
		
	}
	
	public void showInfo(Post selectedPost){
		postName.setText(selectedPost.getName());
		title.setText(selectedPost.getTitle());
		clueForNextPost.setText(selectedPost.getClue().getText());
		tQ.setText(selectedPost.getTask().getQuestion());
		tA.setText(selectedPost.getTask().getAnswer());
		posLat.setText(selectedPost.getPosition().latitudeValue(NonSI.DEGREE_ANGLE)+"");
		posLong.setText(selectedPost.getPosition().longitudeValue(NonSI.DEGREE_ANGLE)+"");
	}

	public Post getSelectedPost() {
		return selectedPost;
	}
}
