package com.habii.treasurehunt.framework.util;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.util.Enumerator;


public enum State implements Enumerator {
	IMPORT(0, "IMPORT", "IMPORT"), EDIT(1, "EDIT", "EDIT"), RUN(2, "RUN", "RUN"), EXPORT(
			3, "EXPORT", "EXPORT"), INIT(4, "INIT", "INIT");

	public static final int IMPORT_VALUE = 0;
	public static final int EDIT_VALUE = 1;
	public static final int RUN_VALUE = 2;
	public static final int EXPORT_VALUE = 3;
	public static final int INIT_VALUE = 4;

	private static final State[] VALUES_ARRAY = new State[] { IMPORT, EDIT,
			RUN, EXPORT, INIT, };

	public static final List<State> VALUES = Collections
			.unmodifiableList(Arrays.asList(VALUES_ARRAY));

	public static State get(String literal) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			State result = VALUES_ARRAY[i];
			if (result.toString().equals(literal)) {
				return result;
			}
		}
		return null;
	}

	public static State getByName(String name) {
		for (int i = 0; i < VALUES_ARRAY.length; ++i) {
			State result = VALUES_ARRAY[i];
			if (result.getName().equals(name)) {
				return result;
			}
		}
		return null;
	}

	public static State get(int value) {
		switch (value) {
		case IMPORT_VALUE:
			return IMPORT;
		case EDIT_VALUE:
			return EDIT;
		case RUN_VALUE:
			return RUN;
		case EXPORT_VALUE:
			return EXPORT;
		case INIT_VALUE:
			return INIT;
		}
		return null;
	}

	private final int value;

	private final String name;

	private final String literal;

	private State(int value, String name, String literal) {
		this.value = value;
		this.name = name;
		this.literal = literal;
	}

	public int getValue() {
		return value;
	}

	public String getName() {
		return name;
	}

	public String getLiteral() {
		return literal;
	}

	@Override
	public String toString() {
		return literal;
	}

}